@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: gtkimcontextscim.cpp,v 1.170.2.1 2005/12/30 08:50:09 suzhe Exp $
+ * $Id: gtkimcontextscim.cpp,v 1.170.2.8 2006/09/29 15:17:48 suzhe Exp $
  */
 
 #define Uses_SCIM_DEBUG
@@ -77,6 +77,8 @@ using namespace scim;
   #define SCIM_KEYBOARD_ICON_FILE            (SCIM_ICONDIR "/keyboard.png")
 #endif
 
+#define SCIM_CONFIG_FRONTEND_GTK_IMMODULE_USE_KEY_SNOOPER  "/FrontEnd/GtkIMModule/UseKeySnooper"
+
 /* Typedef */
 struct _GtkIMContextSCIMImpl
 {
@@ -91,6 +93,7 @@ struct _GtkIMContextSCIMImpl
     gboolean                 use_preedit;
     bool                     is_on;
     bool                     shared_si;
+    bool                     preedit_started;
 
     GtkIMContextSCIMImpl    *next;
 };
@@ -303,6 +306,7 @@ static guint                                            _panel_iochannel_hup_sou
 
 static bool                                             _on_the_spot                = true;
 static bool                                             _shared_input_method        = false;
+static bool                                             _use_key_snooper            = true;
 
 // A hack to shutdown the immodule cleanly even if im_module_exit () is not called when exiting.
 class FinalizeHandler
@@ -536,6 +540,8 @@ gtk_im_context_scim_init (GtkIMContextSCIM      *context_scim,
     context_scim->impl->is_on = FALSE;
     context_scim->impl->shared_si = _shared_input_method;
     context_scim->impl->use_preedit = _on_the_spot;
+    context_scim->impl->preedit_started = false;
+
     context_scim->id = _context_count++;
 
     if (_shared_input_method)
@@ -634,10 +640,17 @@ gtk_im_context_scim_filter_keypress (GtkIMContext *context,
 
     GtkIMContextSCIM *context_scim = GTK_IM_CONTEXT_SCIM (context);
 
-    if (context_scim && context_scim->slave)
-        return gtk_im_context_filter_keypress (context_scim->slave, event);
+    gboolean ret = FALSE;
 
-    return FALSE;
+    if (context_scim) {
+        if (!_snooper_installed)
+            ret = gtk_scim_key_snooper (0, event, 0);
+
+        if (!ret && context_scim->slave)
+            ret = gtk_im_context_filter_keypress (context_scim->slave, event);
+    }
+
+    return ret;
 }
 
 static void
@@ -664,7 +677,8 @@ gtk_im_context_scim_focus_in (GtkIMContext *context)
     if (_focused_ic)
         gtk_im_context_scim_focus_out (GTK_IM_CONTEXT (_focused_ic));
 
-    if (!_snooper_installed) {
+    // Only use key snooper when use_key_snooper option is enabled and a gtk main loop is running.
+    if (_use_key_snooper && !_snooper_installed && gtk_main_level () > 0) {
         _snooper_id = gtk_key_snooper_install ((GtkKeySnoopFunc)gtk_scim_key_snooper, NULL);
         _snooper_installed = true;
     }
@@ -695,6 +709,7 @@ gtk_im_context_scim_focus_in (GtkIMContext *context)
                 context_scim->impl->preedit_string.clear ();
                 context_scim->impl->preedit_attrlist.clear ();
                 context_scim->impl->preedit_caret = 0;
+                context_scim->impl->preedit_started = false;
                 need_cap = true;
                 need_reset = true;
                 need_reg = true;
@@ -707,6 +722,7 @@ gtk_im_context_scim_focus_in (GtkIMContext *context)
                 context_scim->impl->preedit_string.clear ();
                 context_scim->impl->preedit_attrlist.clear ();
                 context_scim->impl->preedit_caret = 0;
+                context_scim->impl->preedit_started = false;
                 attach_instance (context_scim->impl->si);
                 need_cap = true;
                 need_reg = true;
@@ -847,13 +863,16 @@ gtk_im_context_scim_get_preedit_string (GtkIMContext   *context,
                 PangoAttribute *attr;
                 AttributeList::const_iterator i;
                 bool underline = false;
+                bool *attrs_flag = new bool [mbs.length ()];
+
+                memset (attrs_flag, 0, mbs.length () * sizeof (bool));
 
                 for (i = context_scim->impl->preedit_attrlist.begin ();
                      i != context_scim->impl->preedit_attrlist.end (); ++i) {
                     start_index = i->get_start ();
                     end_index = i->get_end ();
 
-                    if (end_index <= wlen && start_index < end_index) {
+                    if (end_index <= wlen && start_index < end_index && i->get_type () != SCIM_ATTR_DECORATE_NONE) {
                         start_index = g_utf8_offset_to_pointer (mbs.c_str (), i->get_start ()) - mbs.c_str ();
                         end_index = g_utf8_offset_to_pointer (mbs.c_str (), i->get_end ()) - mbs.c_str ();
 
@@ -900,15 +919,26 @@ gtk_im_context_scim_get_preedit_string (GtkIMContext   *context,
                             attr->end_index = end_index;
                             pango_attr_list_insert (*attrs, attr);
                         }
+
+                        // Record which character has attribute.
+                        for (guint pos = start_index; pos < end_index; ++pos)
+                            attrs_flag [pos] = 1;
                     }
                 }
 
-                // If there is no underline at all, then draw underline under the whole preedit string.
-                if (!underline) {
-                    attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
-                    attr->start_index = 0;
-                    attr->end_index = mbs.length ();
-                    pango_attr_list_insert (*attrs, attr);
+                // Add underline for all characters which don't have attribute.
+                for (guint pos = 0; pos < mbs.length (); ++pos) {
+                    if (!attrs_flag [pos]) {
+                        guint begin_pos = pos;
+
+                        for (; pos < mbs.length () && !attrs_flag [pos]; ++pos)
+                            NULL;
+
+                        attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
+                        attr->start_index = begin_pos;
+                        attr->end_index = pos;
+                        pango_attr_list_insert (*attrs, attr);
+                    }
                 }
             }
         }
@@ -1371,8 +1401,11 @@ turn_on_ic (GtkIMContextSCIM *ic)
         if (_shared_input_method)
             _config->write (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT), true);
 
-        if (ic->impl->use_preedit && ic->impl->preedit_string.length ())
-            g_signal_emit_by_name(ic, "preedit_changed");
+        if (ic->impl->use_preedit && ic->impl->preedit_string.length ()) {
+            g_signal_emit_by_name(ic, "preedit-start");
+            g_signal_emit_by_name(ic, "preedit-changed");
+            ic->impl->preedit_started = true;
+        }
     }
 }
 
@@ -1393,8 +1426,11 @@ turn_off_ic (GtkIMContextSCIM *ic)
         if (_shared_input_method)
             _config->write (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT), false);
 
-        if (ic->impl->use_preedit && ic->impl->preedit_string.length ())
-            g_signal_emit_by_name(ic, "preedit_changed");
+        if (ic->impl->use_preedit && ic->impl->preedit_string.length ()) {
+            g_signal_emit_by_name(ic, "preedit-changed");
+            g_signal_emit_by_name(ic, "preedit-end");
+            ic->impl->preedit_started = false;
+        }
     }
 }
 
@@ -1926,10 +1962,16 @@ slot_show_preedit_string (IMEngineInstanceBase *si)
     GtkIMContextSCIM *ic = static_cast<GtkIMContextSCIM *> (si->get_frontend_data ());
 
     if (ic && ic->impl && _focused_ic == ic) {
-        if (ic->impl->use_preedit)
-            g_signal_emit_by_name(_focused_ic, "preedit_changed");
-        else
+        if (ic->impl->use_preedit) {
+            if (!ic->impl->preedit_started) {
+                g_signal_emit_by_name(_focused_ic, "preedit-start");
+                ic->impl->preedit_started = true;
+            }
+            if (ic->impl->preedit_string.length ())
+                g_signal_emit_by_name(_focused_ic, "preedit-changed");
+        } else {
             _panel_client.show_preedit_string (ic->id);
+        }
     }
 }
 
@@ -1971,7 +2013,11 @@ slot_hide_preedit_string (IMEngineInstanceBase *si)
             emit = true;
         }
         if (ic->impl->use_preedit) {
-            if (emit) g_signal_emit_by_name(ic, "preedit_changed");
+            if (emit) g_signal_emit_by_name(ic, "preedit-changed");
+            if (ic->impl->preedit_started) {
+                g_signal_emit_by_name(ic, "preedit-end");
+                ic->impl->preedit_started = false;
+            }
         } else {
             _panel_client.hide_preedit_string (ic->id);
         }
@@ -2009,10 +2055,15 @@ slot_update_preedit_caret (IMEngineInstanceBase *si, int caret)
 
     if (ic && ic->impl && _focused_ic == ic && ic->impl->preedit_caret != caret) {
         ic->impl->preedit_caret = caret;
-        if (ic->impl->use_preedit)
-            g_signal_emit_by_name(ic, "preedit_changed");
-        else
+        if (ic->impl->use_preedit) {
+            if (!ic->impl->preedit_started) {
+                g_signal_emit_by_name(_focused_ic, "preedit-start");
+                ic->impl->preedit_started = true;
+            }
+            g_signal_emit_by_name(ic, "preedit-changed");
+        } else {
             _panel_client.update_preedit_caret (ic->id, caret);
+        }
     }
 }
 
@@ -2029,8 +2080,11 @@ slot_update_preedit_string (IMEngineInstanceBase *si,
         ic->impl->preedit_string   = str;
         ic->impl->preedit_attrlist = attrs;
         if (ic->impl->use_preedit) {
-            ic->impl->preedit_caret    = str.length ();
-            g_signal_emit_by_name(ic, "preedit_changed");
+            if (!ic->impl->preedit_started) {
+                g_signal_emit_by_name(_focused_ic, "preedit-start");
+                ic->impl->preedit_started = true;
+            }
+            g_signal_emit_by_name(ic, "preedit-changed");
         } else {
             _panel_client.update_preedit_string (ic->id, str, attrs);
         }
@@ -2073,7 +2127,7 @@ slot_forward_key_event (IMEngineInstanceBase *si,
     if (ic && _focused_ic == ic) {
         GdkEventKey gdkevent = keyevent_scim_to_gdk (ic, key);
         if (!_fallback_instance->process_key_event (key) &&
-            !gtk_im_context_filter_keypress (GTK_IM_CONTEXT (ic), &gdkevent)) {
+            !gtk_im_context_filter_keypress (GTK_IM_CONTEXT (ic->slave), &gdkevent)) {
 
             // To avoid timing issue, we need emit the signal directly, rather than put the event into the queue.
             if (_focused_widget) {
@@ -2240,6 +2294,7 @@ reload_config_callback (const ConfigPointer &config)
 
     _on_the_spot = config->read (String (SCIM_CONFIG_FRONTEND_ON_THE_SPOT), _on_the_spot);
     _shared_input_method = config->read (String (SCIM_CONFIG_FRONTEND_SHARED_INPUT_METHOD), _shared_input_method);
+    _use_key_snooper = config->read (String (SCIM_CONFIG_FRONTEND_GTK_IMMODULE_USE_KEY_SNOOPER), _use_key_snooper);
 
     // Get keyboard layout setting
     // Flush the global config first, in order to load the new configs from disk.
