@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_x11_frontend.cpp,v 1.179 2005/08/16 07:26:54 suzhe Exp $
+ * $Id: scim_x11_frontend.cpp,v 1.179.2.2 2006/09/30 02:18:42 suzhe Exp $
  *
  */
 
@@ -335,8 +335,10 @@ X11FrontEnd::start_helper (int siid, const String &helper_uuid)
 {
     SCIM_DEBUG_FRONTEND(2) << " Start helper, siid=" << siid << " Helper=" << helper_uuid << "\n";
 
-    if (is_focused_ic (siid))
-        m_panel_client.start_helper (m_focus_ic->icid, helper_uuid);
+    X11IC *ic = m_ic_manager.find_ic_by_siid (siid);
+
+    if (validate_ic (ic))
+        m_panel_client.start_helper (ic->icid, helper_uuid);
 }
 
 void
@@ -344,8 +346,10 @@ X11FrontEnd::stop_helper (int siid, const String &helper_uuid)
 {
     SCIM_DEBUG_FRONTEND(2) << " Stop helper, siid=" << siid << " Helper=" << helper_uuid << "\n";
 
-    if (is_focused_ic (siid)) 
-        m_panel_client.stop_helper (m_focus_ic->icid, helper_uuid);
+    X11IC *ic = m_ic_manager.find_ic_by_siid (siid);
+
+    if (validate_ic (ic))
+        m_panel_client.stop_helper (ic->icid, helper_uuid);
 }
 
 void
@@ -353,8 +357,10 @@ X11FrontEnd::send_helper_event (int siid, const String &helper_uuid, const Trans
 {
     SCIM_DEBUG_FRONTEND(2) << " Send helper event, siid=" << siid << " Helper=" << helper_uuid << "\n";
 
-    if (is_focused_ic (siid))
-        m_panel_client.send_helper_event (m_focus_ic->icid, helper_uuid, trans);
+    X11IC *ic = m_ic_manager.find_ic_by_siid (siid);
+
+    if (validate_ic (ic))
+        m_panel_client.send_helper_event (ic->icid, helper_uuid, trans);
 }
 
 bool
@@ -1312,7 +1318,7 @@ X11FrontEnd::ims_preedit_callback_draw (X11IC *ic, const WideString& str, const
     feedback = new XIMFeedback [str.length () + 1];
 
     for (i = 0; i < len; ++i)
-        feedback [i] = XIMUnderline;
+        feedback [i] = 0;
 
     for (i = 0; i < attrs.size (); ++i) {
         attr = 0;
@@ -1326,6 +1332,10 @@ X11FrontEnd::ims_preedit_callback_draw (X11IC *ic, const WideString& str, const
             feedback [j] |= attr;
     }
 
+    for (i = 0; i < len; ++i)
+        if (!feedback [i])
+            feedback [i] = XIMUnderline;
+
     feedback [len] = 0;
 
     pcb.major_code = XIM_PREEDIT_DRAW;
