@@ -1,9 +1,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2004 James Su <suzhe@turbolinux.com.cn>
- * Copyright (c) 2003 James Su <suzhe@turbolinux.com.cn>
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -21,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim.cpp,v 1.44 2004/08/28 03:58:59 suzhe Exp $
+ * $Id: scim.cpp,v 1.51 2005/06/15 00:19:08 suzhe Exp $
  *
  */
 
@@ -29,7 +27,7 @@
 #define Uses_SCIM_IMENGINE_MODULE
 #define Uses_SCIM_BACKEND
 #define Uses_SCIM_CONFIG_PATH
-#define Uses_SCIM_SOCKET_TRANSACTION
+#define Uses_SCIM_TRANSACTION
 #define Uses_C_LOCALE
 #include "scim_private.h"
 #include <scim.h>
@@ -44,8 +42,8 @@ using std::endl;
 
 bool check_socket_frontend ()
 {
-	SocketAddress address;
-	SocketClient client;
+    SocketAddress address;
+    SocketClient client;
 
     uint32 magic;
 
@@ -54,11 +52,11 @@ bool check_socket_frontend ()
     if (!client.connect (address))
         return false;
 
-    if (!scim_socket_trans_open_connection (magic,
-                                            String ("ConnectionTester"),
-                                            String ("SocketFrontEnd"),
-                                            client,
-                                            1000)) {
+    if (!scim_socket_open_connection (magic,
+                                      String ("ConnectionTester"),
+                                      String ("SocketFrontEnd"),
+                                      client,
+                                      1000)) {
         return false;
     }
 
@@ -77,10 +75,6 @@ int main (int argc, char *argv [])
 
     String def_frontend;
     String def_config;
-    String debug_output ("stderr");
-    String debug_mask ("none");
-
-    uint32 verbose_level = 0;
 
     size_t i;
     bool daemon = false;
@@ -181,14 +175,6 @@ int main (int argc, char *argv [])
                  << "  -e, --engines name      Load specified set of IMEngines.\n"
                  << "  -ne,--no-engines name   Do not load those set of IMEngines.\n"
                  << "  -d, --daemon            Run " << argv [0] << " as a daemon.\n"
-#if ENABLE_DEBUG
-                 << "  -v, --verbose           Enable debug info, multiple -v for more.\n"
-                 << "  -o, --output file       Output debug information into file.\n"
-                 << "  -m, --mask mask         Set debug output mask, the valid values are:\n"
-                 << "                          all,main,config,imengine,backend,frontend,module,\n"
-                 << "                          utility,iconv,lookuptable,socket.\n"
-                 << "                          multiple masks can be separated by comma.\n"
-#endif
                  << "  --no-socket             Do not try to start a SCIM SocketFrontEnd daemon.\n"
                  << "  -h, --help              Show this help message.\n";
             return 0;
@@ -222,32 +208,6 @@ int main (int argc, char *argv [])
             continue;
         }
 
-        if (String ("-v") == argv [i] ||
-            String ("--verbose") == argv [i]) {
-            verbose_level += 1;
-            continue;
-        }
-
-        if (String ("-m") == argv [i] ||
-            String ("--mask") == argv [i]) {
-            if (++i >= argc) {
-                cerr << "No argument for option " << argv [i-1] << endl;
-                return -1;
-            }
-            debug_mask = argv [i];
-            continue;
-        }
-
-        if (String ("-o") == argv [i] ||
-            String ("--output") == argv [i]) {
-            if (++i >= argc) {
-                cerr << "No argument for option " << argv [i-1] << endl;
-                return -1;
-            }
-            debug_output = argv [i];
-            continue;
-        }
-
         if (String ("--no-socket") == argv [i]) {
             socket = false;
             continue;
@@ -306,15 +266,12 @@ int main (int argc, char *argv [])
         // If no Socket FrontEnd is running, then launch one.
         // And set manual to false.
         if (!check_socket_frontend ()) {
-            cout << "Launching a SCIM daemon with Socket FrontEnd...\n";
+            cerr << "Launching a SCIM daemon with Socket FrontEnd...\n";
             char *no_stay_argv [] = { "--no-stay", 0 };
             scim_launch (true,
                          def_config,
                          (load_engine_list.size () ? scim_combine_string_list (load_engine_list, ',') : "all"),
                          "socket",
-                         verbose_level,
-                         debug_mask,
-                         debug_output,
                          no_stay_argv);
             manual = false;
         }
@@ -334,16 +291,13 @@ int main (int argc, char *argv [])
         }
     }
 
-    cout << "Launching a SCIM process with " << def_frontend << "...\n";
+    cerr << "Launching a SCIM process with " << def_frontend << "...\n";
 
     // Launch the scim process.
     if (scim_launch (daemon,
                      def_config,
                      (load_engine_list.size () ? scim_combine_string_list (load_engine_list, ',') : "all"),
                      def_frontend,
-                     verbose_level,
-                     debug_mask,
-                     debug_output,
                      new_argv) == 0) {
         if (daemon)
             cerr << "SCIM has been successfully launched.\n";
