@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_utility.cpp,v 1.48 2005/04/30 01:45:03 liuspider Exp $
+ * $Id: scim_utility.cpp,v 1.48.2.4 2006/09/24 16:00:52 suzhe Exp $
  */
 
 #define Uses_SCIM_UTILITY
@@ -694,7 +694,7 @@ static __Language __languages [] = {
     { "ar",    "ar_EG", N_("Arabic"), NULL, NULL },
     { "ar_EG",    NULL, N_("Arabic (Egypt)"), NULL, NULL },
     { "ar_LB",    NULL, N_("Arabic (Lebanon)"), NULL, NULL },
-    { "as",       NULL, N_("Assamese"), NULL, NULL}, 
+    { "as_IN",    NULL, N_("Assamese"), NULL, NULL}, 
     { "az_AZ",    NULL, N_("Azerbaijani"), NULL, NULL },
     { "be_BY",    NULL, N_("Belarusian"), "Беларуская мова", NULL },
     { "bg_BG",    NULL, N_("Bulgarian"), "Български", NULL },
@@ -708,6 +708,7 @@ static __Language __languages [] = {
     { "cy_GB",    NULL, N_("Welsh"), "Cymraeg", NULL },
     { "da_DK",    NULL, N_("Danish"), "dansk", "@euro" },
     { "de_DE",    NULL, N_("German"), "Deutsch", "@euro" },
+    { "dv_MV",    NULL, N_("Divehi"), "ދިވެހިބަސް", NULL },
     { "el_GR",    NULL, N_("Greek"), "ελληνικά", NULL },
     { "en"   , "en_US", N_("English"), "English", NULL },
     { "en_AU",    NULL, N_("English (Australian)"), "Australian English", NULL },
@@ -730,12 +731,14 @@ static __Language __languages [] = {
     { "hi_IN",    NULL, N_("Hindi"), "हिंदी", NULL },
     { "hr_HR",    NULL, N_("Croatian"), "Hrvatski", NULL },
     { "hu_HU",    NULL, N_("Hungarian"), "Magyar", NULL },
+    { "hy_AM",    NULL, N_("Armenian"), "Հայերէն", NULL },
     { "ia"   ,    NULL, N_("Interlingua"), NULL },
     { "id_ID",    NULL, N_("Indonesian"), "Bahasa Indonesia", NULL },
     { "is_IS",    NULL, N_("Icelandic"), NULL, NULL },
     { "it_IT",    NULL, N_("Italian"), "Italiano", "@euro" },
     { "iw_IL",    NULL, N_("Hebrew"), "עברית", NULL },
     { "ja_JP",    NULL, N_("Japanese"), "日本語", ".EUC-JP,.SJIS,.eucJP" },
+    { "ka_GE",    NULL, N_("Georgian"), "ქართული", NULL },
     { "kk_KZ",    NULL, N_("Kazakh"), NULL, NULL },
     { "km",       NULL, N_("Cambodian"), NULL, NULL },
     { "kn_IN",    NULL, N_("Kannada"), "﻿ಕನ್ನಡ", NULL },
@@ -748,18 +751,20 @@ static __Language __languages [] = {
     { "mn_MN",    NULL, N_("Mongolian"), "Монгол", NULL },
     { "mr_IN",    NULL, N_("Marathi"), NULL, NULL },
     { "ms_MY",    NULL, N_("Malay"), "Bahasa Melayu", NULL },
+    { "my_MM",    NULL, N_("Burmese"), "", NULL },
     { "ne_NP",    NULL, N_("Nepali"), NULL, NULL },
     { "nl_NL",    NULL, N_("Dutch"), "Nederlands", "@euro" },
     { "nn_NO",    NULL, N_("Norwegian (nynorsk)"), "Norsk (nynorsk)", NULL },
     { "no_NO",    NULL, N_("Norwegian (bokmal)"), "Norsk (bokmål)", NULL },
     { "or_IN",    NULL, N_("Oriya"), NULL, NULL },
-    { "pa_IN",    NULL, N_("Panjabi"), NULL, NULL },
+    { "pa_IN",    NULL, N_("Punjabi"), NULL, NULL },
     { "pl_PL",    NULL, N_("Polish"), "Polski", NULL },
     { "pt",    "pt_PT", N_("Portuguese"), "Português", NULL },
     { "pt_BR",    NULL, N_("Portuguese (Brazil)"), "Português do Brasil", NULL },
     { "pt_PT",    NULL, N_("Portuguese"), "Português", "@euro" },
     { "ro_RO",    NULL, N_("Romanian"), "Română", NULL },
     { "ru_RU",    NULL, N_("Russian"), "русский", ".koi8r" },
+    { "si_LK",    NULL, N_("Sinhala"), "සිංහල", NULL },
     { "sk_SK",    NULL, N_("Slovak"), "Slovenský", NULL },
     { "sl_SI",    NULL, N_("Slovenian"), "Slovenščina", NULL },
     { "sq_AL",    NULL, N_("Albanian"), "Shqip", NULL },
@@ -1108,7 +1113,7 @@ void scim_daemon ()
         std::cerr << "Error to make SCIM into a daemon!\n";
         return;
     } else if (id > 0) {
-        exit (0);
+        _exit (0);
     }
 
     id = fork ();
@@ -1116,7 +1121,7 @@ void scim_daemon ()
         std::cerr << "Error to make SCIM into a daemon!\n";
         return;
     } else if (id > 0) {
-        exit (0);
+        _exit (0);
     }
 
     return;
